@@ -1,6 +1,6 @@
 package Gtk2::Ex::HyperTextBuffer;
 
-our $VERSION = '0.21';
+our $VERSION = '0.24';
 
 use strict;
 use Gtk2;
@@ -74,12 +74,14 @@ Simple constructor.
 sub INIT_INSTANCE {
 	my $self = shift;
 	$self->{edit_mode_tags} = [];
+	$self->{_edit_mode_string} = '';
 	$self->{undo} = [];
 	$self->{redo} = [];
 	$self->signal_connect(mark_set => sub {
 			my ($self, $iter, $mark) = @_;
+			my $name = $mark->get_name;
 			$self->set_edit_mode_from_cursor()
-				if $mark->get_name eq 'insert';
+				if defined($name) and $name eq 'insert';
 		} );
 	$self->signal_connect_after(insert_text => \&on_insert);
 	$self->signal_connect(delete_range => \&on_delete_range);
@@ -212,9 +214,7 @@ sub _insert_blocks {
 			my ($type, $meta) = @$_[0, 1];
 			my $tag;
 			if ($type eq 'image') {
-				my %data = %$meta; # copy
-				my @arg = delete @data{qw/src width height/};
-				$self->insert_image_from_file(@arg, \%data);
+				$self->insert_image_from_file(%$meta);
 				next;
 			}
 			elsif ($type eq 'link') {
@@ -231,11 +231,6 @@ sub _insert_blocks {
 				@{$self->{edit_mode_tags}} = ();
 			}
 			elsif ($type eq 'Page' or $type eq 'Para') {
-				if ($$meta{base}) {
-					$self->{base} = $$meta{base};
-					$self->{base} =~ s#^file://##;
-					#warn "base set to: $self->{base}\n";
-				}
 				$self->_insert_blocks(@$_[2 .. $#$_]); # recurs
 			}
 			else { die "Unknown tag in parse tree: $type" }
@@ -247,47 +242,39 @@ sub _insert_blocks {
 	}
 }
 
-=item C<insert_image_from_file(FILENAME, WIDTH, HEIGHT, DATA, ITER)>
+=item C<insert_image_from_file(%PARAM)>
 
-Insert an image in the buffer at the cursor
-or at ITER if ITER is specified.
+Insert an image in the buffer at the cursor. The params should at least contain
+'file' and optionally 'src', 'width' and 'height'.
 
-WIDTH and HEIGHT are optional arguments. If both are set the image
-is resized to these dimensions. If one of both is zero or undefined
-the image is scaled with fixed ratio.
-
-DATA is an optional hash reference that is attached to the image object.
+The 'file' param is used as source for the image. If both 'width' and 'height'
+are set the image is resized to these dimensions. If one of both is zero or
+undefined the image is scaled with fixed ratio. All other params are attached
+to the image as data field.
 
 Use C<insert_blocks()> if you want this action to be un-doable. 
 
 =cut
 
 sub insert_image_from_file {
-	my ($self, $src, $w, $h, $data, $iter) = @_;
-	($w, $iter) = (undef, $w) if @_ == 3 and ref $w;
-	my $file = $self->{base} ? File::Spec->rel2abs($src, $self->{base}) : $src;
-	#warn "file: $src, base: $self->{base} => $file\n";
+	my ($self, %param) = @_;
+	die "BUG: image without 'file' attribute" unless length $param{file};
 
 	my $pixbuf = undef;
-	if (-f $file and -r _) {
-		unless ($w || $h) {
-			eval { $pixbuf =
-				Gtk2::Gdk::Pixbuf->new_from_file($file) }
-		}
-		else {
-			eval { $pixbuf =
-				Gtk2::Gdk::Pixbuf->new_from_file_at_size(
-					$file, $w || -1, $h || -1) }
-		}
-	}
-	$pixbuf ||= Gtk2::Image->new->render_icon('gtk-missing-image', 'dialog')->copy;
-	die "Could not insert image: $file\n" unless $pixbuf;
+	my ($w, $h) = @param{'width', 'height'};
+	eval {
+		$pixbuf = ($w || $h)
+			? Gtk2::Gdk::Pixbuf->new_from_file_at_size(
+				$param{file}, $w || -1, $h || -1 )
+			: Gtk2::Gdk::Pixbuf->new_from_file(
+				$param{file} ) ;
+	} if -f $param{file} and -r _ ;
+	$pixbuf ||= Gtk2::Image->new->render_icon(
+			'gtk-missing-image', 'dialog' )->copy;
+	die "Could not insert image: $param{file}\n" unless $pixbuf;
 	
-	$pixbuf->{image_src} = $src;
-	$pixbuf->{image_file} = $file;
-	$pixbuf->{image_data} = $data;
-	$pixbuf->{resized} = ($w || $h) ? [$w, $h] : 0;
-	$iter ||= $self->get_iter_at_mark( $self->get_insert );
+	$pixbuf->{image_data} = \%param;
+	my $iter ||= $self->get_iter_at_mark( $self->get_insert );
 
 	#my $mark = $self->create_mark(undef, $iter, 0);
 	#$mark->set_visible(1);
@@ -337,11 +324,7 @@ sub get_parse_tree {
 				$image->forward_chars(length $1);
 				my $pixbuf = $image->get_pixbuf;
 				if ($pixbuf) {
-					my $meta = { src => $pixbuf->{image_src} };
-					@$meta{qw/width height/} = @{$pixbuf->{resized}}
-						if $pixbuf->{resized};
-					%$meta = (%{$pixbuf->{image_data}}, %$meta)
-						if $pixbuf->{image_data};
+					my $meta = $pixbuf->{image_data};
 					push @{$stack[-1]}, ['image', $meta];
 				}
 				$image->forward_char;
@@ -413,7 +396,7 @@ sub get_parse_tree {
 	# Filter out empty tags
 	@$root = grep {
 		defined($_) and
-		(ref($_) and ($$_[0] !~ /^(link|image)/)) 
+		(ref($_) and ($$_[0] ne 'image')) 
 			? (@$_ > 2)
 			: length($_)
 	} @$root;
