@@ -4,14 +4,16 @@ package Zim::Formats;
 # TODO: hash with aliases from names to modules
 
 use strict;
+use Carp;
 use constant {
 	READ   => 1,	# we can input this format
 	WRITE  => 2,	# we can output this format
 	NATIVE => 4,	# we can read/write instead of import/export
 };
-use File::BaseDir qw/xdg_data_home xdg_data_dirs/;
+use File::BaseDir 0.03 qw/data_home data_dirs/;
+use Zim::Utils;
 
-our $VERSION = '0.23';
+our $VERSION = '0.24';
 
 =head1 NAME
 
@@ -99,12 +101,11 @@ sub list_templates {
 	$format = lc $format;
 
 	my %templates;
-	for (xdg_data_home(), xdg_data_dirs()) {
-		my $dir = File::Spec->catdir($_, 'zim', 'templates', $format);
-		next unless -d $dir;
-		for my $f (Zim::File->list_dir($dir)) {
+	for my $dir (data_dirs('zim', 'templates', $format)) {
+		$dir = dir($dir);
+		for my $f ($dir->list) {
 			$f =~ /(.*?)(\.|$)/;
-			$templates{$1} = "$dir/$f"
+			$templates{$1} = $dir->file($f)
 				unless defined $templates{$1};
 		}
 	}
@@ -179,56 +180,64 @@ sub _dump_nodes {
 
 =over 4
 
-=item C<parse_link(LINK, PAGE)>
+=item C<parse_link(LINK, PAGE, NO_RESOLVE)>
 
-Returns the link type and link target for LINK. The type can be 'page',
+Returns the link type and link target for LINK. The type can be e.g. 'page',
 'file', 'mail' or 'man' - for urls the protocol is used as type.
 
-The target returned is usually the same as LINK but can be different
-for example for interwiki links.
+The link target is e.g. a fully specified page name, a fully specified path,
+or an url. For file:// urls the path is returned. For mail a 'mailto:' url is
+returned.
+
+If NO_RESOLVE is set we only check the type, but do not try to fully specify
+the target.
 
 =cut
 
 sub parse_link {
-	my (undef, $link, $page) = @_;
-	if ($link =~ m#^(\w[\w\+\-\.]+)\?(.*)#) { # interwiki link
-		my $l = Zim->interwiki_lookup($1, $2);
-		$link = $l if defined($l);
-		# when lookup failed we pass on trough
-		# this ensures that things like man?zim will also work
-		# in special cases a interwiki may in turn return
-		# a "key?page" string, now "key" is the type
+	my (undef, $link, $page, $no_resolve) = @_;
+	#warn "Parsing link: $link ($page)\n";
+
+	# Interwiki links e.g. foo?bar:baz
+	if ($link =~ m#^(\w[\w\+\-\.]+)\?(.*)#) {
+		my ($name, $page) = ($1, $2);
+		my $url = Zim->interwiki_lookup($name, $page);
+		$link = $url if defined($url);
+		# Returns e.g. http://, file:// or zim:// on success
+		# When lookup fails we pass on trough, this ensures 
+		# that things like "man?zim" will also work.
 	}
-	#warn "Parsing link: $link\n";
 	
 	my $type;
-	$_ = $link;
-	if    (m#^(\w+[\w\+\-\.]+)://#) { # urls
+	# URLS e.g. file://foo or http://foo
+	if ($link =~ m#^(\w+[\w\+\-\.]+)://#) {
 		$type = $1;
-		$link = Zim::File->parse_uri($link) if $type eq 'file';
+		$link = $page->resolve_file($link)
+			if $type eq 'file' and ! $no_resolve;
 	}
-	elsif (m#^mailto:|^\S+\@\S+\.\w+$#) { # email
-		$link =~ s#^(mailto:)?#mailto:#;
+	# mail e.g. mailto:foo@bar or foo@bar
+	elsif ($link =~ m#^mailto:|^\S+\@\S+\.\w+$#) { #
 		$type = 'mail';
+		$link =~ s#^(mailto:)?#mailto:#;
 	}
-	elsif (m#^(\w[\w\+\-\.]+)\?(.*)#) { # type?string
+	# Any other special types e.g. man?test (passtrough from interwiki)
+	elsif ($link =~ m#^(\w[\w\+\-\.]+)\?(.*)#) {
 		($type, $link) = ($1, $2);
 	}
-	else { $type = (m#/#) ? 'file' : 'page' } # file or page
-	
-	if ($type eq 'page') {
-		$link =~ s#^\.:*#$page:#; # sub-page
-		$link = Zim::Store->clean_name($link, 'NOT_ABS');
+	# Files and directories - anything containing a '/'
+	elsif ($link =~ m#/#) {
+		$type = 'file';
+		$link = $page->resolve_file($link) unless $no_resolve;
 	}
-	elsif ($type eq 'file') {
-		if ($link =~ /^\//) { # document root  ( ignores file:// )
-			my $root = $page->{root}->config->{file_root} || '';
-			$link = $root . $link;
-		}
-		$link = Zim::File->abs_path(
-			$link, $page->{properties}{base} );
+	# Page - default type
+	else {
+		$type = 'page';
+		$link = Zim::Store->clean_name($link, 'RELATIVE');
+		$link =~ s#^\.:*#$page:#; # sub-page - FIXME does this belong here?
+		$link = $page->resolve_name($link) unless $no_resolve;
 	}
 
+	#warn "type: $type link: $link\n";
 	return $type, $link;
 }
 
