@@ -2,35 +2,40 @@ package Zim::GUI::Component;
 
 use strict;
 use vars qw/$AUTOLOAD/;
+use Gtk2;
 use Carp;
 use Encode;
-use File::BaseDir qw/xdg_data_home xdg_data_dirs xdg_data_files/;
-use Zim::File;
+use File::BaseDir 0.03 qw/data_dirs data_files/;
+use Zim::Events;
 use Zim::Utils;
   
-our $VERSION = '0.20';
-  
-($Zim::ICON) = xdg_data_files('pixmaps', 'zim.png');
+our $VERSION = '0.24';
+
+our @ISA = qw/Zim::Events/;
+
+$Zim::ICON = data_files('pixmaps', 'zim.png');
 warn "WARNING: Could not find 'zim.png', is \$XDG_DATA_DIRS set properly ?\n"
 	unless length $Zim::ICON;
   
 { # load all icons in pixmaps/zim
- 	my @dirs = grep -d $_, map "$_/pixmaps/zim",
- 		xdg_data_home, xdg_data_dirs;
-	my @files = ($Zim::ICON);
- 	for my $d (@dirs) {
-		push @files, map $d.'/'.$_, Zim::File->list_dir($d);
+	my @files = length($Zim::ICON) ? ($Zim::ICON) : ();
+	for my $dir (data_dirs('pixmaps', 'zim')) {
+		$dir = dir($dir);
+		push @files, map $dir->file($_), $dir->list;
 	}
 	return unless @files;
 
 	my $factory = Gtk2::IconFactory->new;
 	$factory->add_default;
+	my %icons;
 	for my $f (@files) {
 		my $pixbuf = Gtk2::Gdk::Pixbuf->new_from_file($f);
 		my $icon_set = Gtk2::IconSet->new_from_pixbuf($pixbuf);
 		my $n = $f;
 		$n =~ s/.*[\/\\]//;
 		$n =~ s/\..*//;
+		next if exists $icons{$n};
+		$icons{$n} = $f;
 		#warn "Icon: $n => $f\n";
 		$factory->add($n => $icon_set);
 	}
@@ -81,6 +86,37 @@ Called by the constructor, to be overloaded.
 
 sub init {};
 
+=item C<init_settings(NAME, \%DEFAULTS)>
+
+Will initialise a hash C<$$self{settings}> which is linked to section NAME
+in the config file. Intended for use by plugin modules.
+
+When the whole section is missing it will be initialized with %DEFAULTS.
+
+=item C<init_state(NAME, \%DEFAULTS)>
+
+Like C<init_settings()> but initializes C<$$self{state}> and links to
+file with state variables, not config file.
+
+=cut
+
+sub init_settings { _init_config_hash('settings', @_) }
+
+sub init_state    { _init_config_hash('state', @_)    }
+
+sub _init_config_hash {
+	my ($hash, $self, $name, $defaults) = @_;
+	unless ($$self{app}{$hash}{$name}) {
+		$$self{app}{$hash}{$name} = $defaults || {};
+		$$self{app}->SaveSettings if $hash eq 'settings';
+	}
+	$$self{$hash} = $$self{app}{$hash}{$name}
+		unless $self eq $$self{app};
+		# Exception in case we are called on top level object
+	croak "BUG: not a hash: $name\n" unless ref $$self{$hash} eq 'HASH';
+	return $$self{app}{$hash}{$name};
+}
+
 =item C<widget()>
 
 Returns the "top_widget" attribute. This should be the toplevel widget
@@ -114,7 +150,7 @@ sub AUTOLOAD {
 		}
 		else { return $self->$AUTOLOAD() }
 	}
-	else {
+	elsif (ref $_[0]) {
 		my $self = shift;
 		my $class = ref $self;
 		croak "No such method: $class::$AUTOLOAD"
@@ -122,6 +158,7 @@ sub AUTOLOAD {
 			and    $self->{widget}->can($AUTOLOAD);
 		return $self->{widget}->$AUTOLOAD(@_);
 	}
+	else { croak "No such method: $AUTOLOAD" }
 }
 
 =back
@@ -130,15 +167,15 @@ sub AUTOLOAD {
 
 =over 4
 
-=item C<add_actions($ACTIONS, $TYPE, $NAME)>
+=item C<add_actions(ACTIONS, TYPE, FUNCTION)>
 
 Add plain text action descriptions to the interface.
 
 TYPE can be C<undef>, "menu", "toggle" or "radio".
 
-NAME is an optonal group name. This is used to set a single
+FUNCTION is an optional function name. This is used to set a single
 callback for all actions. If NAME is not defined the callback
-for each action will be the name of the action.
+function for each action will be of the same name as the action.
 
 =cut
 
@@ -157,8 +194,6 @@ sub add_actions {
 	my @actions;
 	for (grep /\S/, split /\n/, $actions) {
 		my @a = map {($_ eq '.') ? undef : $_} split /\t+/, $_;
-		$a[2] = __($a[2]);
-		$a[4] = __($a[4]);
 		my $n = $name || $a[0] ;
 		push @a, \&{$class.'::on_'.$n} unless $type eq 'menu';
 		push @a, '0' if $type eq 'toggle';
@@ -180,7 +215,7 @@ sub _add_radio_group {
 	my @actions;
 	for (@l) {
 		my @a = map {($_ eq '.') ? undef : $_} split /\t+/, $_;
-		push @a, $val++;
+		$a[5] = $val++;
 		push @actions, \@a;
 	}
 	#use Data::Dumper; warn "Actions (radio): ", Dumper \@actions;
@@ -234,10 +269,10 @@ sub actions_set_active {
 	my ($self, %actions) = @_;
 	for my $name (keys %actions) {
 		my $action = $self->{actions}->get_action($name);
-        unless ($action) {
-            carp "BUG: no such action: $name\n";
-            next;
-        }
+		unless ($action) {
+			carp "BUG: no such action: $name\n";
+			next;
+		}
 		$action->set_active($actions{$name} ? 1 : 0);
 	}
 }
@@ -302,8 +337,8 @@ Show a window showing the documentation. PAGE is optional.
 
 sub ShowHelp {
 	my ($self, $page) = @_;
-	my @args = ('--doc', $page ? $page : ());
-	$self->exec_new_window(@args);
+	$self->call_daemon('open', '_doc_', $page) and return;
+	$self->exec_new_window('--doc', $page);
 }
 
 =back
@@ -312,15 +347,30 @@ sub ShowHelp {
 
 =over 4
 
-=item  C<exec_new_window(..)>
+=item C<call_daemon(@ARGS)>
+
+Echo arguments to the daemon. See L<Zim::GUI::Daemon>.
+Returns FALSE when no daemon listening.
+
+=item C<exec_new_window(@ARGS)>
 
 Executes a new process for $0, this gives a detached window.
 Any arguments are passed on to the new process.
 
 =cut
 
+sub call_daemon {
+	my ($self, @args) = @_;
+	return 0 if $$self{app}{no_daemon};
+	warn "## Call daemon: @args\n";
+	eval { Zim::GUI::Daemon->do(@args) };
+	warn "## No daemon listening\n" if $@;
+	return $@ ? 0 : 1 ;
+}
+
 sub exec_new_window {
 	my ($self, @args) = @_;
+	unshift @args, '--no-daemon' if $$self{app}{no_daemon};
 	Zim::Utils->run($^X, $0, @args);
 }
 
@@ -341,15 +391,38 @@ Page name didn't go through cleanup, so does not need to be valid.
 sub check_page_input {
 	my ($self, $name) = @_;
 	return undef unless length $name;
-	my ($t, $l) = Zim::Formats->parse_link($name, $self->{app}{page});
+	my ($t, $l) = Zim::Formats->parse_link(
+		$name, $self->{app}{page}, 'NO_RESOLVE');
 		# default parse_link(), not page dependent
 	return $l if $t eq 'page' and length $l;
 	$self->error_dialog(
-		"Not a valid page name: $name",
+		__("Not a valid page name: {name}", name => $name),
 		"Parsing gives type '$t' for page '$name'" );
 	return undef;
 }
 
+=item C<check_file_input(PATH)>
+
+Resolves user input to an absolute path. When users specify a path it could
+be either be relative to document dir or document dir. This method tries to
+figure it out and return the right path. File does not need to exit.
+Returns a file object.
+
+=cut
+
+sub check_file_input {
+	my ($self, $path) = @_;
+	return Zim::FS::File->new($path) unless $$self{app}{page};
+
+	# A file that looks absolute can still be relative to the
+	# document root. If it exists, it is probably really absolute.
+	# Make it an uri to force resolve_file() to take is absolute.
+	$path = Zim::FS->path2uri($path)
+		if Zim::FS->is_abs_path($path) and -f $path;
+	
+	return $$self{app}{page}->resolve_file($path);
+}
+
 =item C<decode_uri_list(TEXT)>
 
 Method to decode data in the C<text/uri-list> format.
@@ -439,14 +512,13 @@ sub new_prompt {
 	my ($self, $title, $fields, $data, $stock, $string, $text) = @_;
 	
 	## Setup dialog
-	$title =~ s/(\s+\-)?\s*zim\s*$//i;
 	$title .= ' - Zim';
 	my $dialog = Gtk2::Dialog->new(
 		$title, $self->{app}{window},
 		[qw/modal destroy-with-parent no-separator/],
 		'gtk-cancel'  => 'cancel',
 	);
-	$dialog->set_resizable(0);
+	#$dialog->set_resizable(0);
 	#$dialog->vbox->set_border_width(12); # FIXME
 	$dialog->set_icon($self->{app}{window}->get_icon);
 	
@@ -479,8 +551,15 @@ FIELDS is an array ref giving the order of the input fields.
 DATA is a hash ref containing definitions of the input fields.
 The key is the name used in FIELDS, the value an array ref with a label text,
 a data type and a value.
-At the moment only the "string", "page", "file" and "dir" data types are
-treated special, all other will be ignored silently.
+
+At the moment only the "string", "int", "page", "file" and "dir" data types
+are treated special, all other will be ignored silently.
+
+The "file" and "dir" types can have a 4th item being the dir to open
+when the value is empty and the "Browse" button is clicked.
+
+The "int" type has a range and a step size, so the 4th argument should be
+a array ref with 3 numbers: min, max and step.
 
 DIALOG is optional and will be used to connect signals.
 
@@ -502,48 +581,76 @@ sub new_form {
 		my $align = Gtk2::Alignment->new(0,0.5, 0,0);
 		$align->add($label);
 		
-		my $entry = Gtk2::Entry->new();
-		$entry->set_text($f[2]) if defined $f[2];
-		push @entries, $entry;
-		
-		$table->attach_defaults($align, 0,1, $i,$i+1);
+		$table->attach($align, 0,1, $i,$i+1, ['fill'],[], 0,0);
 
+		my $entry;
 		if ($f[1] eq 'file' or $f[1] eq 'dir') {
+			# for file and dir input we add a "Browse" button
+			$entry = Gtk2::Entry->new();
+			$entry->set_text($f[2]) if defined $f[2];
+
 			my $hbox = Gtk2::HBox->new(0,3);
 			$hbox->add($entry);
 
 			my $is_dir = ($f[1] eq 'dir');
-			my $button = Gtk2::Button->new('_Browse...');
+			my $button = Gtk2::Button->new(__('_Browse...')); #. button
 			$button->signal_connect( clicked => sub {
+				# open filechooser dialog
 				my $val = $entry->get_text();
-				$val = File::Spec->rel2abs('./___')
-					if $is_dir and ! length $val;
-					# force current dir instead of parent
+				$val = $self->check_file_input($val);
+				$val = $f[3] if $val !~ /\S/;
 				$val = $self->filechooser_dialog($val, $is_dir);
-				$entry->set_text($val);
+				$entry->set_text($val) if defined $val;
+			} );
+			$entry->signal_connect('notify::sensitive' => sub {
+				# sync sensitive state of button with entry
+				my $s = $entry->is_sensitive;
+				$button->set_sensitive($s);
 			} );
 			$hbox->pack_start($button, 0,1,0);
 
 			$table->attach_defaults($hbox, 1,2, $i,$i+1);
 		}
-		elsif ($f[1] eq 'page') {
-			$self->set_page_completion($entry);
+		elsif ($f[1] eq 'int') {
+			# for integers we use a spin button input
+			my @r = @{$f[3]};
+			$entry = Gtk2::SpinButton->new_with_range(@r);
+			$entry->set_value($f[2]);
 			$table->attach_defaults($entry, 1,2, $i,$i+1);
 		}
+		elsif ($f[1] eq 'page') {
+			# for page we add page completion to the entry
+			$entry = Gtk2::Entry->new();
+			$entry->set_text($f[2]) if defined $f[2];
+			$table->attach_defaults($entry, 1,2, $i,$i+1);
+			$self->set_page_completion($entry);
+		}
 		elsif ($f[1] eq 'password') {
+			# of course a password should not be inputted
+			# in plain text
+			$entry = Gtk2::Entry->new();
+			$entry->set_text($f[2]) if defined $f[2];
 			$entry->set_visibility(0);
 			$table->attach_defaults($entry, 1,2, $i,$i+1);
 		}
+#		elsif ($f[1] eq 'dropdown') {
+#			my $combo = ...;
+#			$combo->set_text($_) for @$f[2];
+#		}
 		else {
+			# string or unknown type
+			$entry = Gtk2::Entry->new();
+			$entry->set_text($f[2]) if defined $f[2];
 			$table->attach_defaults($entry, 1,2, $i,$i+1);
 		}
+		push @entries, $entry;
 	}
 	
 	for my $i (0 .. $#entries - 1) {
 		$entries[$i]->signal_connect(
 			activate => sub { $entries[$i+1]->grab_focus } );
 	}
-	if ($dialog) {
+	if ($dialog and @entries) {
 		$entries[-1]->signal_connect(
 			activate => sub { $dialog->response('ok') } );
 	}
@@ -570,20 +677,20 @@ sub set_page_completion {
 
 sub _update_completion {
 	my ($entry, $self) = @_;
-	return unless $self->{app}{repository} and $self->{app}{page};
+	return unless $self->{app}{notebook} and $self->{app}{page};
 	my $ns = $entry->get_text;
 	$ns =~ s/[^:]+$//;
 	return if defined $entry->{_ns} and $entry->{_ns} eq $ns;
 	$entry->{_ns} = $ns;
 	
 	my $_ns = length($ns)
-		? $self->{app}{repository}->resolve_namespace($ns)
+		? $self->{app}{notebook}->resolve_namespace($ns)
 		: $self->{app}{page}->namespace() ;
 		#warn "Complete namespace: $_ns\n";
 	
 	my $model = $entry->get_completion->get_model;
 	$model->clear;
-	for ($self->{app}{repository}->list_pages($_ns)) {
+	for ($self->{app}{notebook}->list_pages($_ns)) {
 		s/_/ /g;
 		my $iter = $model->append();
 		$model->set($iter, 0 => $ns.$_);
@@ -612,13 +719,16 @@ sub run_prompt {
 
 =item C<prompt_question(TITLE, TYPE, TEXT, BUTTONS ..., TIME)>
 
-Runs a dialog displaying TEXT
+TYPE can either be 'error', 'warning', 'question', 'info' or C<undef>.
+
+Runs a dialog displaying TEXT To comform with the Gnome HIG,
+the text should consist of a primary text in bold, possibly followed by
+an empty line and secondary text in normal style. Please make sure the
+text ends with a question if you expect a response.
 
 BUTTONS is a list of array references, each containing a name, a stock item
 name and/or text. The id of the button that was pressed is returned.
 
-TYPE can either be 'error', 'warning', 'question', 'info' or C<undef>.
-
 TIME is an optional argument, it gives a timeout in seconds. This is used
 for popups that can popup while the user is typing to prevent accidental
 triggering of a accelerator.
@@ -626,16 +736,15 @@ triggering of a accelerator.
 =cut
 
 sub prompt_question {
-	my ($self, $title, $type, $text, @buttons) = @_;
+	my ($self, $type, $text, @buttons) = @_;
 	my $time = pop @buttons unless ref $buttons[-1];
 	
-	$title =~ s/(\s+\-)?\s*zim\s*$//i;
-	$title .= ' - Zim';
 	my $dialog = Gtk2::Dialog->new(
-		$title, $self->{app}{window},
+		undef, $self->{app}{window}, # HIG says no title
 	       	[qw/modal destroy-with-parent no-separator/],
 	);
 	$dialog->set_resizable(0);
+	$dialog->set_border_width(6); # border from HIG
 	$dialog->set_icon($self->{app}{window}->get_icon);
 
 	my @button_widgets;
@@ -649,28 +758,43 @@ sub prompt_question {
 		push @button_widgets, $button;
 	}
 	
-	my $hbox = Gtk2::HBox->new(0,12);
-	$hbox->set_border_width(12);
+	my $hbox = Gtk2::HBox->new(0,12); # spacing from HIG
+	$hbox->set_border_width(6); # border from HIG
 	$dialog->vbox->pack_start($hbox, 0,0,0);
 
 	if (defined $type) {
 		my $image = Gtk2::Image->new_from_stock(
 			"gtk-dialog-$type", 'dialog' );
-		$image->set_alignment(0.0, 0.5); # valign=top
+		$image->set_alignment(0.5, 0.0); # HIG says valign top
 		$hbox->pack_start($image, 0,0,0);
 	}
 	if (defined $text) {
+		$text =~ s/\n*$/\n/; # HIG says one empty line
 		my $label = Gtk2::Label->new($text);
-		$label->set_use_markup(1);
-		$label->set_selectable(1);
-		$label->set_alignment(0.0, 0.0); # align left top corner
+		$label->set_use_markup(1); # enable bold text
+		$label->set_alignment(0.0, 0.0); # left top
 		$hbox->add($label);
 	}
 
 	$dialog->show_all;
 	if ($time) {
-		Glib::Timeout->add( $time*1000,
-			sub { $_->set_sensitive(1) for @button_widgets; 0 } );
+		my $label = Gtk2::Label->new($time.' sec.');
+		$label->set_state('insensitive');
+		$label->set_alignment(0.9, 0.5); # right center
+		$label->show;
+		$dialog->vbox->add($label);
+		Glib::Timeout->add( 1000, sub {
+			$time--;
+			if ($time > 0) {
+				$label->set_text($time. ' sec.');
+				1; # continue
+			}
+			else {
+				$label->set_text('');
+				$_->set_sensitive(1) for @button_widgets;
+				0; # last
+			}
+		} );
 	}
 	else { $_->set_sensitive(1) for @button_widgets }
 	my $id = $dialog->run;
@@ -721,47 +845,83 @@ sub error_dialog {
 	return undef;
 }
 
-=item C<filechooser_dialog(FILE)>
+=item C<filechooser_dialog(PATH, WANT_DIR, TITLE)>
 
-Ask the user for a filename. FILE is the suggested filename.
+Show the user a dialog to browse the file system and choose a file or
+directory. PATH is the suggested file or directory. WANT_DIR is a
+boolean that controls whether this dialog selects a file or a dir.
+TITLE is an optional title for the dialog.
+
+If PATH is not defined, either the last directory used by a filechooser
+dialog from this GUI Component, the current document root or
+document dir will be opened. To control which directory opens, set the
+attribute C<$$self{filechooser_dir}>. This attribute will be set to the
+lastest directory after the dialog is closed.
+
+Returns an absolute path or undef.
 
 =cut
 
 sub filechooser_dialog {
-	my ($self, $file, $dir, $title) = @_;
+	my ($self, $file, $want_dir, $title) = @_;
+	my ($doc_dir, $doc_dir_root, $doc_root) = $$self{app}{page}
+		? ( $$self{app}{page}->document_dir,
+		    $$self{app}{notebook}->document_dir,
+		    $$self{app}{notebook}->document_root )
+		: (undef, undef) ;
+		# need to check page here because this method is also
+		# used by e.g. NotebookDialog
 	
+	# construct filechooser dialog
 	my $dialog;
-	$title ||= $dir ? 'Select Folder' : 'Select File' ;
-	# if (Gtk2->CHECK_VERSION(2, 4, 0) and $Gtk2::VERSION >= 1.040) {
+	$title ||= $want_dir
+		? __('Select Folder') : __('Select File') ; #. dialog title
+	$title .= ' - Zim';
 	$dialog = Gtk2::FileChooserDialog->new(
 		$title, $self->{app}{window}, 'open',
 		'gtk-cancel' => 'cancel',
 		'gtk-ok'     => 'ok'
 	);
-	# }
-	#else { # old & ugly interface
-	#	$dialog = Gtk2::FileSelection->new($title);
-	#}
 	$dialog->set_icon($self->{app}{window}->get_icon);
-	$dialog->set_action('select-folder') if $dir;
+	$dialog->set_action('select-folder') if $want_dir;
+
+	# add links to special directories in side pane
+	for (grep {defined $_} $doc_root, $doc_dir_root, $doc_dir) {
+		my $folder = Zim::FS->localize($_);
+		$dialog->add_shortcut_folder($folder) if -e $folder; 
+			# dies for some Gtk version if folder does not exist
+	}
+	
+	# set the file or dir we want the dialog to open
 	if (defined $file) {
-		$file = Zim::File->localize(
-				Zim::File->abs_path($file) );
+		# opens the dir for file (file can be a dir itself)
+		$file = Zim::FS->localize(file($file));
 		$dialog->set_filename($file);
 	}
-	elsif (defined $self->{app}{page}) {
-		my $dir = $self->{app}{page}->properties->{base};
+	else {
+		# set the directory to start browsing
+		my ($dir) = grep {-d $_} grep {defined $_}
+			$$self{filechooser_dir},
+			$doc_root, $doc_dir, $doc_dir_root ;
 		if (defined $dir) {
-			$dir = Zim::File->localize(
-				Zim::File->abs_path($dir) );
+			$dir = Zim::FS->localize(dir($dir));
 			$dialog->set_current_folder($dir);
 		}
 	}
-	$dialog->signal_connect('response', sub {
-		$file = $_[1] eq 'ok' ? $dialog->get_filename : undef;
-		$dialog->destroy;
-	} );
-	$dialog->run;
+
+	# run the dialog
+	$dialog->show_all;
+	my $re = $dialog->run;
+	if ($re eq 'ok') {
+		$file = $dialog->get_filename;
+		$file = Zim::FS->abs_path($file);
+			# abs_path to un-localize
+		my $dir = $dialog->get_current_folder;
+		$$self{filechooser_dir} = Zim::FS->abs_path($dir);
+			# abs_path to un-localize
+	}
+	else { $file = undef }
+	$dialog->destroy;
 
 	return $file;
 }
