@@ -6,7 +6,7 @@ Svn support
 
 =head1 VERSION
 
-0.2
+0.24
 
 =head1 DESCRIPTION
 
@@ -25,7 +25,7 @@ modify it under the same terms as Perl itself.
 
 
 my $app  = Zim::GUI->current;
-my $dir = $app->{repository}->{dir};
+my $dir = $app->{notebook}->{dir};
 
 
 # do you have subversion tools installed?
@@ -40,28 +40,28 @@ $app->{actions}->add_actions( [
 		'gtk-go-down',		# Stock icon name
 		'SVN checkout',		# Menu text
 		undef,			# Keybinding
-		'Checkout zim root from SVN repository',	# Tooltip text
+		'Checkout notebook from SVN repository',	# Tooltip text
 		\&on_SvnCheckout,	# Callback code
 	],
 	[	'SvnUpdate',		# Name - used below in xml
 		'gtk-refresh',		# Stock icon name
 		'SVN update',		# Menu text
 		undef,			# Keybinding
-		'Update zim root from SVN repository',	# Tooltip text
+		'Update notebook from SVN repository',	# Tooltip text
 		\&on_SvnUpdate,	# Callback code
 	],
 	[	'SvnCommit',		# Name - used below in xml
 		'gtk-apply',		# Stock icon name
 		'SVN commit',		# Menu text
 		undef,			# Keybinding
-		'Commit zim root to SVN repository',	# Tooltip text
+		'Commit notebook to SVN repository',	# Tooltip text
 		\&on_SvnCommit,	# Callback code
 	],
 	[	'SvnImport',		# Name - used below in xml
 		'gtk-go-up',		# Stock icon name
 		'SVN import',		# Menu text
 		undef,			# Keybinding
-		'Import zim root to SVN repository',	# Tooltip text
+		'Import notebook to SVN repository',	# Tooltip text
 		\&on_SvnImport,	# Callback code
 	],
 	[	'SvnCleanup',		# Name - used below in xml
@@ -108,7 +108,7 @@ sub is_recent {
 	my $local_revision = 0;
 	my $revision = 0;
 	my $page = $app->{page};
-	my $rep = $page->{repository};
+	my $rep = $page->{store};
 	return unless $rep->can('file');
 	my $file = $rep->file( $page );
 	my $filepath = $file->path;
@@ -161,7 +161,7 @@ sub is_wc {
 				'SvnImport' => 1,
 				'SvnCleanup' => 0
 				);
-		$app->message("Directory $dir is not SVN working copy")
+		warn "Directory $dir is not SVN working copy";
 	}
 	# if dir is SVN working copy 
 	else {
@@ -172,7 +172,7 @@ sub is_wc {
 				'SvnImport' => 0,
 				'SvnCleanup' => 1
 				);
-		$app->message("Directory $dir is SVN working copy")
+		warn "Directory $dir is SVN working copy";
 	}
 }
 
@@ -272,12 +272,18 @@ sub on_SvnCommit {
 			}
 		}
 
-		if (system ("svn ci -m $log_msg $dir &>/dev/null")) {
-			$app->error_dialog("Failed commit changes to SVN repository!");
+		# Save comment in a temp file to allow spaces and quotes inside.
+		my $comment = Zim::FS->tmp_file('SubversionComment.txt');
+		$comment->write($log_msg);
+		my $null = Zim::FS->devnull;
+	 
+		if (system ("svn ci -F \"$comment\" \"$dir\" &>$null") == 0) {
+			warn "Commit successfull.\n";
 		}
 		else {
-			$app->message('Commit successfull.');
+			$app->error_dialog("Failed commit changes to SVN repository!");
 		}
+		$comment->remove;
 	}
 
 
@@ -290,7 +296,7 @@ sub on_SvnUpdate {
 	else {
 		$app->RBIndex();
 		$app->Reload();
-		$app->message('Update successfull.');
+		warn 'Update successfull.';
 	}
 
 }
